#! /usr/bin/env python3
#
# Copyright (c) 2023 ZKM | Hertz-Lab
# Dan Wilcox <dan.wilcox@zkm.de>
#
# BSD Simplified License.
# For information on usage and redistribution, and for a DISCLAIMER OF ALL
# WARRANTIES, see the file, "LICENSE.txt," in this distribution.
#
# This code has been developed at ZKM | Hertz-Lab as part of „The Intelligent
# Museum“ generously funded by the German Federal Cultural Foundation.
#
# References:
# * https://github.com/eclipse/paho.mqtt.python
# * https://github.com/attwad/python-osc

import sys, argparse, json

import paho.mqtt.client as mqtt
from pythonosc.udp_client import SimpleUDPClient
from pythonosc import osc_message_builder

import skeeter

mqtt_host = "localhost"
mqtt_port = 1883
mqtt_topic = "test"

osc_host = "127.0.0.1"
osc_port = 7788

verbose = True

##### parser

parser = argparse.ArgumentParser(description="OSC <- MQTT relay server")
parser.add_argument(
    "host", type=str, nargs="?", metavar="HOST",
    default="", help="MQTT server host name or IP address, optionally append port HOST:PORT")
parser.add_argument(
    "topic", type=str, nargs="?", metavar="TOPIC",
    default="", help="MQTT topic")
parser.add_argument(
    "-a", "--address", action="store", dest="address",
    default="127.0.0.1", help="OSC send address, default: 127.0.0.1")
parser.add_argument(
    "-p", "--port", action="store", dest="port",
    default=7788, type=int, help="OSC send port, default: 7788")
parser.add_argument(
    "-v", "--verbose", action="store_true", dest="verbose",
    help="enable verbose printing")

##### mqtt

# mqtt connection callback
def mqtt_on_connect(client, userdata, flags, rc):
    if verbose:
        print(f"mqtt: connected with result code {rc}")

# mqtt message callback
def mqtt_on_message(client, userdata, msg):
    if verbose:
        print(f"{msg.topic} {str(msg.payload)}")
    if not msg.payload or msg.payload == "": return
    try:
        payload = json.loads(msg.payload.decode("utf-8"))
    except json.JSONDecodeError as exc:
        print(f"mqtt: could not parse message to osc: {exc}", file=sys.stderr)
        return
    address,args = skeeter.json_to_osc(payload)
    if address == None and args == None:
        print("mqtt: could not parse message to osc", file=sys.stderr)
        return
    message = osc_message_builder.OscMessageBuilder(address=address)
    for arg in args:
        message.add_arg(arg)
    sender.send(message.build())

##### main

# parse config
args = parser.parse_args()
if args.host == "" or args.topic == "":
        print("MQTT HOST & TOPIC required", file=sys.stderr)
        sys.exit(1)
if ':' in args.host: # parse optional HOST:PORT
    s = args.host.split(":")
    try: # strip last component
        mqtt_port = int(s[-1])
    except:
        print(f"invalid port {s[-1]} in MQTT HOST {args.host}", file=sys.stderr)
        sys.exit(-1)
    # join everything else, in case of domain ie. http://
    mqtt_host = ":".join(s[:-1])
mqtt_topic = args.topic
osc_host = args.address
osc_port = args.port
verbose = args.verbose
args = None

# mqtt client
client = mqtt.Client()
client.on_connect = mqtt_on_connect
client.on_message = mqtt_on_message

# osc sender
sender = SimpleUDPClient(osc_host, osc_port)

# loop
print(f"osc {osc_host}:{osc_port} <- mqtt {mqtt_host}:{mqtt_port} {mqtt_topic}")
try:
    client.connect(mqtt_host, mqtt_port, 60)
    client.subscribe(mqtt_topic)
    client.loop_forever()
except KeyboardInterrupt:
    pass
except Exception as exc:
    print(exc, file=sys.stderr)
