#! /usr/bin/env python3
#
# Copyright (c) 2023 ZKM | Hertz-Lab
# Dan Wilcox <dan.wilcox@zkm.de>
#
# BSD Simplified License.
# For information on usage and redistribution, and for a DISCLAIMER OF ALL
# WARRANTIES, see the file, "LICENSE.txt," in this distribution.
#
# This code has been developed at ZKM | Hertz-Lab as part of „The Intelligent
# Museum“ generously funded by the German Federal Cultural Foundation.
#
# References:
# * https://github.com/eclipse/paho.mqtt.python
# * https://github.com/attwad/python-osc

import asyncio, argparse, signal, sys, json

import paho.mqtt.client as mqtt
from pythonosc.dispatcher import Dispatcher
from pythonosc.osc_server import AsyncIOOSCUDPServer

import skeeter

mqtt_host = "localhost"
mqtt_port = 1883
mqtt_topic = "test"

osc_host = "127.0.0.1"
osc_port = 7777

verbose = True

##### parser

parser = argparse.ArgumentParser(description="OSC -> MQTT relay server")
parser.add_argument(
    "host", type=str, nargs="?", metavar="HOST",
    default="", help="MQTT server host name or IP address, optionally append port HOST:PORT")
parser.add_argument(
    "topic", type=str, nargs="?", metavar="TOPIC",
    default="", help="MQTT topic")
parser.add_argument(
    "-a", "--address", action="store", dest="address",
    default="127.0.0.1", help="OSC receive address, default: 127.0.0.1")
parser.add_argument(
    "-p", "--port", action="store", dest="port",
    default=7777, type=int, help="OSC receive port, default: 7777")
parser.add_argument(
    "-v", "--verbose", action="store_true", dest="verbose",
    help="enable verbose printing")

##### mqtt

# mqtt connection callback
def mqtt_on_connect(client, userdata, flags, rc):
    if verbose:
        print(f"mqtt: connected with result code {rc}")

##### osc

# osc message callback, send osc messages as json
# see oscparser.py for conversion details
def osc_received(address, *args):
    if verbose:
        print(f"{address} {list(args)}")
    data = skeeter.osc_to_json(address, list(args))
    if data == None:
        print("osc: could not parse message to json")
        return
    try:
        client.publish(mqtt_topic, payload=json.dumps(data), qos=0, retain=False)
    except Exception as exc:
        print(exc, file=sys.stderr)

##### signal

# signal handler for nice exit
def sigint_handler():
    asyncio.get_running_loop().stop()

##### main

# parse config
args = parser.parse_args()
if args.host == "" or args.topic == "":
        print("MQTT HOST & TOPIC required", file=sys.stderr)
        sys.exit(1)
if ':' in args.host: # parse optional HOST:PORT
    s = args.host.split(":")
    try: # strip last component
        mqtt_port = int(s[-1])
    except:
        print(f"invalid port {s[-1]} in MQTT HOST {args.host}", file=sys.stderr)
        sys.exit(-1)
    # join everything else, in case of domain ie. http://
    mqtt_host = ":".join(s[:-1])
mqtt_topic = args.topic
osc_host = args.address
osc_port = args.port
verbose = args.verbose
args = None

# signal handling
loop = asyncio.get_event_loop()
loop.add_signal_handler(signal.SIGINT, sigint_handler)

# mqtt client
client = mqtt.Client()
client.on_connect = mqtt_on_connect

# osc receiver
dispatcher = Dispatcher()
dispatcher.set_default_handler(osc_received)
receiver = AsyncIOOSCUDPServer((osc_host, osc_port), dispatcher, loop)
loop.run_until_complete(receiver.create_serve_endpoint())

# wait for osc receiver to exit
print(f"osc {osc_host}:{osc_port} -> mqtt {mqtt_host}:{mqtt_port} {mqtt_topic}")
try:
    client.connect(mqtt_host, mqtt_port, 60)
    loop.run_forever()
except KeyboardInterrupt:
    pass
except Exception as exc:
    print(exc, file=sys.stderr)
finally:
    client.disconnect()
